/*******************************************************************************
 * Copyright (c) 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.models.common.facades.behavioral;

/**
 * 
 * A target invocation is an action that is a particular communication between
 * the component that the action belongs to and a target component.  This can be
 * visualized as a method invocation on a SystemUnderTest.  The specifics of the
 * invocation are captured in the ActionProperties of the target invocation.
 * 
 * @author marcelop
 * @author psun
 * @since 
 */
public interface ITargetInvocation extends IAction
{
	/**
	 * Returns the target component for the invocation.
	 * 
	 * <p>This reference is unidirectional.
	 * 
	 * @return IDeployableComponent
	 * 
	 */
	public IDeployableComponent getTarget();

	/**
	 * Sets the target component for the invocation.
	 * 
	 * <p>This reference is unidirectional.
	 * 	 
	 * @param deployableComponent
	 * 
	 */
	public void setTarget(IDeployableComponent deployableComponent);
}
