/*
 * Created on May 15, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.eclipse.hyades.models.common.facades.behavioral.test;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import org.eclipse.hyades.models.common.facades.behavioral.IAction;
import org.eclipse.hyades.models.common.facades.behavioral.IBlock;
import org.eclipse.hyades.models.common.facades.behavioral.ICondition;
import org.eclipse.hyades.models.common.facades.behavioral.IDecision;
import org.eclipse.hyades.models.common.facades.behavioral.IImplementor;
import org.eclipse.hyades.models.common.facades.behavioral.ILoop;
import org.eclipse.hyades.models.common.facades.behavioral.IStub;
import org.eclipse.hyades.models.common.facades.behavioral.ISystemUnderTest;
import org.eclipse.hyades.models.common.facades.behavioral.ITargetInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITest;
import org.eclipse.hyades.models.common.facades.behavioral.ITestCase;
import org.eclipse.hyades.models.common.facades.behavioral.ITestInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITestSuite;
import org.eclipse.hyades.models.common.facades.behavioral.IVerificationPoint;
import org.eclipse.hyades.models.common.facades.behavioral.impl.HyadesFactory;

/**
 * @author psun
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ValidateFacadeImpl extends TestCase {

	Resource resource = null;
	String xmiFile = "TempEMFFile.xmi";

	public ValidateFacadeImpl(String name)
	{
		super(name);
	}

	public static Test suite()
	{
		TestSuite junitTestSuite = new TestSuite("EMF Facade Impl Validation");
		
		junitTestSuite.addTest(new ValidateFacadeImpl("testTestSuiteCreation"));
		junitTestSuite.addTest(new ValidateFacadeImpl("testTestSuiteModification"));
		junitTestSuite.addTest(new ValidateFacadeImpl("testAddAction"));
		
		return junitTestSuite;
	}
	
	public void setUp()
	{ 
		if(resource == null)
		{
			ResourceSet resourceSet = new ResourceSetImpl();
			File file = new File(xmiFile);
			URI fileURI = URI.createFileURI(file.getAbsolutePath());
			System.out.println("file is " + file.getAbsolutePath());
			Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
			Map m = reg.getExtensionToFactoryMap();
			m.put("xmi", new XMIResourceFactoryImpl());
			 
			resourceSet.getResourceFactoryRegistry();
			resource = resourceSet.createResource(fileURI);
		}

	}
		
	public void tearDown()
	{ 
	}
	
	public void testTestSuiteCreation()
	{
		ITestSuite testSuite = createTestSuite();
		ITestCase testCase = createTestCase();
		IImplementor implementor = createImplementor(testSuite);
		IStub stub = createStub(testSuite);
		implementor.setName("Test Driver 1");
		implementor.setDescription("A test driver");
		implementor.setLocation("main");
		implementor.setResource("C:\\temp\\testDriver1.java");
		ISystemUnderTest systemUnderTest = createSystemUnderTest();
		systemUnderTest.setName("System Under Test 1");
		systemUnderTest.setDescription("A generic system under test");
		systemUnderTest.setLocation("");
		systemUnderTest.setResource("C:\\temp\\systemUnderTest1.java");
		stub.setName("Stub 1");
		stub.setDescription("A sample stub");
		stub.setResource("c:\\temp\\stub1.java");
		stub.setResource("");
		
		
		testSuite.setName("Test Suite 1");
		testSuite.setDescription("A simple Test Suite with one of everythiing");
		testSuite.getITestCases().add(0, testCase);
		testSuite.getISystemsUnderTest().add(0, systemUnderTest);
		testSuite.getIStubs().add(0, stub);
		
		// check object		
		assertEquals(testSuite.getName(), "Test Suite 1");
		assertEquals(testSuite.getDescription(), "A simple Test Suite with one of everythiing");
		assertEquals(testSuite.getITestCases().get(0), testCase);
		assertEquals(testSuite.getIStubs().get(0), stub);
		assertEquals(testSuite.getISystemsUnderTest().get(0), systemUnderTest);		
		assertEquals(testSuite.getImplementor(), implementor);

		// save and reload into another object				
		save(testSuite);
		ITestSuite reloadedTestSuite = (ITestSuite)load();
		
		// check reloaded object
		assertEquals(reloadedTestSuite.getName(), "Test Suite 1");
		assertEquals(reloadedTestSuite.getDescription(), "A simple Test Suite with one of everythiing");
		assertEquals(reloadedTestSuite.getITestCases().get(0), testCase);
		assertEquals(reloadedTestSuite.getIStubs().get(0), stub);
		assertEquals(reloadedTestSuite.getISystemsUnderTest().get(0), systemUnderTest);		
		assertEquals(reloadedTestSuite.getImplementor(), implementor);
	}
	
	public void testTestSuiteModification()
	{
		ITestSuite testSuite = (ITestSuite)load();
		
		// change some values
		

		testSuite.setName("Changed Test Suite");
		testSuite.setDescription("A changed Test Suite");
		
		
		// add/remove some elements
		
		ISystemUnderTest systemUnderTest2 = createSystemUnderTest();
		ISystemUnderTest systemUnderTest3 = createSystemUnderTest();
		systemUnderTest2.setName("System Under Test 2");
		systemUnderTest2.setDescription("A generic system under test");
		systemUnderTest2.setLocation("");
		systemUnderTest2.setResource("C:\\temp\\systemUnderTest2.java");

		systemUnderTest3.setName("System Under Test 3");
		systemUnderTest3.setDescription("A generic system under test");
		systemUnderTest3.setLocation("");
		systemUnderTest3.setResource("C:\\temp\\systemUnderTest3.java");

		
		testSuite.getISystemsUnderTest().remove(0);
		testSuite.getISystemsUnderTest().add(systemUnderTest2);
		testSuite.getISystemsUnderTest().add(systemUnderTest3);

		IStub newStub = createStub(testSuite);
		newStub.setName("Test Component 2");
		newStub.setDescription("A second stub");
		newStub.setLocation("");
		newStub.setResource("C:\\temp\\testComponent2.java");
		testSuite.getIStubs().add(newStub);
		
		// check object
		assertEquals(testSuite.getName(), "Changed Test Suite");
		assertEquals(testSuite.getDescription(), "A changed Test Suite");
		assertEquals(testSuite.getISystemsUnderTest().size(), 2);
		assertEquals(testSuite.getISystemsUnderTest().get(0), systemUnderTest2);
		assertEquals(testSuite.getISystemsUnderTest().get(1), systemUnderTest3);
		assertEquals(((ISystemUnderTest)testSuite.getISystemsUnderTest().get(0)).getName(), "System Under Test 2");
		assertEquals(testSuite.getIStubs().get(1), newStub);
		
		// save and reload into another object		
		save(testSuite);
		ITestSuite reloadedTestSuite = (ITestSuite)load();
		
		// check reloaded object
		assertEquals(reloadedTestSuite.getName(), "Changed Test Suite");
		assertEquals(reloadedTestSuite.getDescription(), "A changed Test Suite");
		assertEquals(reloadedTestSuite.getISystemsUnderTest().get(0), systemUnderTest2);
		assertEquals(reloadedTestSuite.getISystemsUnderTest().get(1), systemUnderTest3);
		assertEquals(((ISystemUnderTest)reloadedTestSuite.getISystemsUnderTest().get(0)).getName(), "System Under Test 2");		
		assertEquals(reloadedTestSuite.getIStubs().get(1), newStub);
	}
	
	public void testAddAction()
	{
		ITestSuite testSuite = (ITestSuite)load();
		IImplementor implementor = (IImplementor)testSuite.getImplementor();
		IBlock block = implementor.getBlock();	

		// Create anothee test suite to invoke.
		ITestSuite testSuite2 = createTestSuite();
		IImplementor implementor2 = createImplementor(testSuite2);

		// create some actions
		IVerificationPoint actionVP = createVerificationPoint();
		actionVP.setName("VP 1");
		actionVP.setDescription("A generic VP");
/*		ITargetInvocation actionTargetInvocation = createTargetInvocation();
		actionTargetInvocation.setName("Target Invocation 1");
		actionTargetInvocation.setDescription("A simple target invocation");
		actionTargetInvocation.setTarget((IDeployableComponent)testSuite.getISystemsUnderTest().get(0));
*/
		ITestInvocation actionTestInvocation = createTestInvocation();
		actionTestInvocation.setName("Test Invocation 1");
		actionTestInvocation.setDescription("A simple test invocation");
		actionTestInvocation.setInvokedTest(testSuite2);
		
		// create a decision
		
		// TODO: Does passing block make sense
		IDecision decision = createDecision(block);
		ICondition condition = decision.getCondition();
		condition.setConstraint("i>5");
		condition.setName("Condition 1");
		condition.setDescription("Less than 5");
		
		// Build the "then" clause
		decision.getSuccessBlock().getActions().add(actionTestInvocation);
		decision.getSuccessBlock().getActions().add(actionVP);

		// Build the "else" clause 
//		decision.getFailureBlock().getActions().add(actionTargetInvocation);
		
		implementor.getBlock().getActions().add(0, decision);

		// check object
		IImplementor tempImplementor = 	(IImplementor)testSuite.getImplementor();
		IAction tempAction = (IAction)tempImplementor.getBlock().getActions().get(0);
		assertEquals(tempAction, decision);
		assertEquals(((IDecision)tempAction).getCondition().getConstraint(), "i>5");
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().size(), 2);
//		assertEquals(((IDecision)tempAction).getFailureBlock().getActions().size(), 1);
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().get(0), actionTestInvocation);
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().get(1), actionVP);
//		assertEquals(((IDecision)tempAction).getFailureBlock().getActions().get(0), actionTargetInvocation);
		
		// save and reload into another object		
		save(testSuite);
		ITestSuite reloadedTestSuite = (ITestSuite)load();

		// check object
		
		tempImplementor = 	(IImplementor)reloadedTestSuite.getImplementor();
		tempAction = (IAction)tempImplementor.getBlock().getActions().get(0);
		assertEquals(tempAction, decision);
		assertEquals(((IDecision)tempAction).getCondition().getConstraint(), "i>5");
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().size(), 2);
//		assertEquals(((IDecision)tempAction).getFailureBlock().getActions().size(), 1);
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().get(0), actionTestInvocation);
		assertEquals(((IDecision)tempAction).getSuccessBlock().getActions().get(1), actionVP);
//		assertEquals(((IDecision)tempAction).getFailureBlock().getActions().get(0), actionTargetInvocation);
		
	}
	
	private ITestSuite createTestSuite()
	{
		return HyadesFactory.INSTANCE.createTestSuite(resource);	
	}
	
	private ITestCase createTestCase()
	{
		return HyadesFactory.INSTANCE.createTestCase(resource);
	}
	
	private IImplementor createImplementor(ITest implementedTest)
	{
		return HyadesFactory.INSTANCE.createImplementor(resource, implementedTest);	
	}

	private IStub createStub(ITest implementedTest)
	{
		return HyadesFactory.INSTANCE.createStub(resource);	
	}
		
	private ISystemUnderTest createSystemUnderTest()
	{
		return HyadesFactory.INSTANCE.createSystemUnderTest(resource);	
	}
	
	private IDecision createDecision(IBlock block)
	{
		return HyadesFactory.INSTANCE.createDecision(block);
	}

	private ILoop createLoop(IBlock block)
	{
		return HyadesFactory.INSTANCE.createLoop(block);	
	}
		
	private IVerificationPoint createVerificationPoint()
	{
		return HyadesFactory.INSTANCE.createVerificationPoint();
	}
		
	private ITargetInvocation createTargetInvocation()
	{
		return HyadesFactory.INSTANCE.createTargetInvocation();
	}

	private ITestInvocation createTestInvocation()
	{
		return HyadesFactory.INSTANCE.createTestInvocation(resource);
	}
		
			
	private void save(Object obj)
	{
		resource.getContents().add(obj);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		try {
			resource.save(options);
		}
		catch ( Exception exc )
		{
		}
	}
	
	private Object load()
	{
		ResourceSet resourceSet = new ResourceSetImpl();
		resource = resourceSet.getResource(URI.createURI(xmiFile), true);
		return resource.getContents().get(0);				
	}
}
