/*******************************************************************************
 * Copyright (c) 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.models.common.facades.behavioral.impl;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.resource.Resource;

import org.eclipse.hyades.models.common.configuration.impl.CFGInstanceImpl;
import org.eclipse.hyades.models.common.configuration.impl.Common_ConfigurationFactoryImpl;
import org.eclipse.hyades.models.common.facades.behavioral.IBlock;
import org.eclipse.hyades.models.common.facades.behavioral.IDecision;
import org.eclipse.hyades.models.common.facades.behavioral.IExternalImplementor;
import org.eclipse.hyades.models.common.facades.behavioral.IImplementor;
import org.eclipse.hyades.models.common.facades.behavioral.ILoop;
import org.eclipse.hyades.models.common.facades.behavioral.IStub;
import org.eclipse.hyades.models.common.facades.behavioral.ISystemUnderTest;
import org.eclipse.hyades.models.common.facades.behavioral.ITargetInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITest;
import org.eclipse.hyades.models.common.facades.behavioral.ITestCase;
import org.eclipse.hyades.models.common.facades.behavioral.ITestInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITestSuite;
import org.eclipse.hyades.models.common.facades.behavioral.IVerificationPoint;
import org.eclipse.hyades.models.common.fragments.Common_Behavior_FragmentsFactory;
import org.eclipse.hyades.models.common.fragments.impl.BVRCombinedFragmentImpl;
import org.eclipse.hyades.models.common.fragments.impl.BVRInteractionConstraintImpl;
import org.eclipse.hyades.models.common.fragments.impl.BVRInteractionImpl;
import org.eclipse.hyades.models.common.fragments.impl.BVRInteractionOperandImpl;
import org.eclipse.hyades.models.common.fragments.impl.Common_Behavior_FragmentsFactoryImpl;
import org.eclipse.hyades.models.common.interactions.impl.BVRExecutionOccurrenceImpl;
import org.eclipse.hyades.models.common.interactions.impl.BVRLifelineImpl;
import org.eclipse.hyades.models.common.interactions.impl.Common_Behavior_InteractionsFactoryImpl;
import org.eclipse.hyades.models.common.testprofile.Common_TestprofileFactory;
import org.eclipse.hyades.models.common.testprofile.TPFBehavior;
import org.eclipse.hyades.models.common.testprofile.TPFTest;
import org.eclipse.hyades.models.common.testprofile.TPFTestComponent;
import org.eclipse.hyades.models.common.testprofile.impl.Common_TestprofileFactoryImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFBehaviorImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFSUTImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFTestCaseImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFTestComponentImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFTestSuiteImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFValidationActionImpl;

/**
 * Factory that creates the Hyades implementation of the simple facade. 
 * 
 * @author marcelop
 * @author psun
 * @since 0.0.1
 */
public class HyadesFactory
{
	public static final HyadesFactory INSTANCE = new HyadesFactory();
	
	public ITestSuite createTestSuite(Resource resource)
	{
		TPFTestSuiteImpl testSuiteImpl = (TPFTestSuiteImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestSuite();
		resource.getContents().add(testSuiteImpl);
		return testSuiteImpl;
	}

	// TODO: Check this
	public ITestCase createTestCase(Resource resource)
	{
		TPFTestCaseImpl testCaseImpl = (TPFTestCaseImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestCase();
		resource.getContents().add(testCaseImpl);
		return testCaseImpl;	
	}

	public IImplementor createImplementor(Resource resource, ITest implementedTest)
	{
		TPFBehaviorImpl testBehavior = (TPFBehaviorImpl) Common_TestprofileFactoryImpl.eINSTANCE.createTPFBehavior();

		if ( implementedTest != null )
		{
			associateImplementor(testBehavior, implementedTest);
		}

		// Since this behavior is modeled, create the mandatory Interaction.
		BVRInteractionImpl testInteraction = (BVRInteractionImpl) Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRInteraction();
		testBehavior.setInteraction(testInteraction);
		
		// Create lifeline
		BVRLifelineImpl selfLifeline = (BVRLifelineImpl)Common_Behavior_InteractionsFactoryImpl.eINSTANCE.createBVRLifeline();
				
		// Add lifeline to interaction
		testInteraction.getLifelines().add(selfLifeline);
		
		// Create instance
		CFGInstanceImpl instance = (CFGInstanceImpl) Common_ConfigurationFactoryImpl.eINSTANCE.createCFGInstance();
		instance.setLifeline(selfLifeline);
		selfLifeline.getInstances().add(instance);
		
		// Associate instance with Deployable (testsuite)
		TPFTestSuiteImpl testSuite;
		if ( implementedTest instanceof ITestSuite )
		{
			testSuite = (TPFTestSuiteImpl) implementedTest;
		}
		else if ( implementedTest instanceof ITestCase )
		{
			testSuite = (TPFTestSuiteImpl)((ITestCase)implementedTest).getOwner();
		}
		else
		{
			throw new IllegalArgumentException("ITest must be either a ITestCase or a ITestSuite.");
		}
		
		instance.setDeployableType(testSuite);
		EList contents = resource.getContents(); 
		contents.add(testBehavior);
		contents.add(testInteraction);
		contents.add(selfLifeline);
		contents.add(instance);
		 		
		return (TPFBehaviorImpl)testBehavior;
	}

	public IExternalImplementor createExternalImplementor(Resource resource, ITest implementedTest)
	{
		TPFBehaviorImpl testBehavior = (TPFBehaviorImpl) Common_TestprofileFactoryImpl.eINSTANCE.createTPFBehavior();
		resource.getContents().add(testBehavior);

		if ( implementedTest != null )
		{
			associateImplementor(testBehavior, implementedTest);
		}

		return (TPFBehaviorImpl)testBehavior;
	}
	
	protected void associateImplementor( TPFBehaviorImpl behavior, ITest implementedTest )
	{
		if(implementedTest instanceof TPFTest)
		{
			TPFTest test = (TPFTest)implementedTest;
			test.setBehavior(behavior);
		}
	}

	public IStub createStub(Resource resource)
	{
		TPFTestComponent testComponent = Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestComponent();
		resource.getContents().add(testComponent);
		TPFBehavior behavior = Common_TestprofileFactory.eINSTANCE.createTPFBehavior();
		testComponent.getBehaviors().add(behavior);

		return (TPFTestComponentImpl)testComponent;
	}

	// TODO: Under Construction		
	public IDecision createDecision(IBlock block)
	{	
		BVRCombinedFragmentImpl testDecisionImpl = (BVRCombinedFragmentImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRCombinedFragment();
	
		block.getActions().add(testDecisionImpl);

		BVRInteractionOperandImpl successOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		BVRInteractionOperandImpl failureOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		
		testDecisionImpl.getInteractionOperands().add(0, successOperand);
		testDecisionImpl.getInteractionOperands().add(1, failureOperand);
		
		// Create an empty constraint for this decision.  
		// The caller will need to add a condition to this constraint 
		BVRInteractionConstraintImpl testConditionImpl = (BVRInteractionConstraintImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRInteractionConstraint();
		successOperand.setInteractionConstraint(testConditionImpl);

		return testDecisionImpl;
	}

	// TODO: Under Construction
	public ILoop createLoop(IBlock block)
	{
		BVRCombinedFragmentImpl testLoopImpl = (BVRCombinedFragmentImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRCombinedFragment();
	
		block.getActions().add(testLoopImpl);
	
		BVRInteractionOperandImpl loopOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		testLoopImpl.getInteractionOperands().add(loopOperand);

		// Create an empty constraint for this loop.  
		// The caller will need to add a condition to this constraint 
		BVRInteractionConstraintImpl testConditionImpl = (BVRInteractionConstraintImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRInteractionConstraint();
		loopOperand.setInteractionConstraint(testConditionImpl);

		
		return testLoopImpl;
	}

	public ISystemUnderTest createSystemUnderTest(Resource resource)
	{
		TPFSUTImpl testSUTImpl = (TPFSUTImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFSUT();
		resource.getContents().add(testSUTImpl);
		return testSUTImpl;
	}

	public IVerificationPoint createVerificationPoint()
	{
		TPFValidationActionImpl testVPImpl = (TPFValidationActionImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFValidationAction();
		return testVPImpl;
	}

	// TODO: Needs to be implemented
	public ITargetInvocation createTargetInvocation()
	{
		return null;
	}

	public ITestInvocation createTestInvocation(Resource resource)
	{
		BVRExecutionOccurrenceImpl execOccur = (BVRExecutionOccurrenceImpl)Common_Behavior_InteractionsFactoryImpl.eINSTANCE.createBVRExecutionOccurrence();
		TPFBehaviorImpl invokedBehavior = (TPFBehaviorImpl) Common_TestprofileFactoryImpl.eINSTANCE.createTPFBehavior();
		resource.getContents().add(invokedBehavior);
		execOccur.setOtherBehavior(invokedBehavior);
		
		return execOccur;
	}

	
}